@@ -67,18 +67,18 @@ use while_loop::convert_while_loop_to_asm;
 /// The [HllAsmSet] contains either a contract ABI and corresponding ASM, a script's main
 /// function's ASM, or a predicate's main function's ASM. ASM is never generated for libraries,
 /// as that happens when the library itself is imported.
-pub enum HllAsmSet<'sc> {
+pub enum HllAsmSet {
     ContractAbi {
-        data_section: DataSection<'sc>,
-        program_section: AbstractInstructionSet<'sc>,
+        data_section: DataSection,
+        program_section: AbstractInstructionSet,
     },
     ScriptMain {
-        data_section: DataSection<'sc>,
-        program_section: AbstractInstructionSet<'sc>,
+        data_section: DataSection,
+        program_section: AbstractInstructionSet,
     },
     PredicateMain {
-        data_section: DataSection<'sc>,
-        program_section: AbstractInstructionSet<'sc>,
+        data_section: DataSection,
+        program_section: AbstractInstructionSet,
     },
     // Libraries do not generate any asm.
     Library,
@@ -87,18 +87,18 @@ pub enum HllAsmSet<'sc> {
 /// An [AbstractInstructionSet] is a set of instructions that use entirely virtual registers
 /// and excessive moves, with the intention of later optimizing it.
 #[derive(Clone)]
-pub struct AbstractInstructionSet<'sc> {
-    ops: Vec<Op<'sc>>,
+pub struct AbstractInstructionSet {
+    ops: Vec<Op>,
 }
 
 /// "Realized" here refers to labels -- there are no more organizational
 /// ops or labels. In this struct, they are all "realized" to offsets.
-pub struct RealizedAbstractInstructionSet<'sc> {
-    ops: Vec<RealizedOp<'sc>>,
+pub struct RealizedAbstractInstructionSet {
+    ops: Vec<RealizedOp>,
 }
 
-impl<'sc> RealizedAbstractInstructionSet<'sc> {
-    fn allocate_registers(self) -> InstructionSet<'sc> {
+impl RealizedAbstractInstructionSet {
+    fn allocate_registers(self) -> InstructionSet {
         // Eventually, we will use a cool graph-coloring algorithm.
         // For now, just keep a pool of registers and return
         // registers when they are not read anymore
@@ -133,14 +133,14 @@ impl<'sc> RealizedAbstractInstructionSet<'sc> {
 
 /// An [InstructionSet] is produced by allocating registers on an [AbstractInstructionSet].
 #[derive(Clone)]
-pub struct InstructionSet<'sc> {
-    ops: Vec<AllocatedOp<'sc>>,
+pub struct InstructionSet {
+    ops: Vec<AllocatedOp>,
 }
 
-type Data<'sc> = Literal<'sc>;
-impl<'sc> AbstractInstructionSet<'sc> {
+type Data = Literal;
+impl AbstractInstructionSet {
     /// Removes any jumps that jump to the subsequent line
-    fn remove_sequential_jumps(&self) -> AbstractInstructionSet<'sc> {
+    fn remove_sequential_jumps(&self) -> AbstractInstructionSet {
         let mut buf = vec![];
         for i in 0..self.ops.len() - 1 {
             if let Op {
@@ -190,9 +190,9 @@ impl<'sc> AbstractInstructionSet<'sc> {
     /// and one to replace the labels in the organizational ops
     fn realize_labels(
         self,
-        data_section: &DataSection<'sc>,
+        data_section: &DataSection,
         _namespace: &AsmNamespace,
-    ) -> RealizedAbstractInstructionSet<'sc> {
+    ) -> RealizedAbstractInstructionSet {
         let mut label_namespace: HashMap<&Label, u64> = Default::default();
         let mut counter = 0;
         for op in &self.ops {
@@ -357,7 +357,7 @@ fn virtual_register_is_never_accessed_again(
 }
 
 /// helper function to check if a label is used in a given buffer of ops
-fn label_is_used<'sc>(buf: &[Op<'sc>], label: &Label) -> bool {
+fn label_is_used(buf: &[Op], label: &Label) -> bool {
     buf.iter().any(|Op { ref opcode, .. }| match opcode {
         Either::Right(OrganizationalOp::Jump(ref l)) if label == l => true,
         Either::Right(OrganizationalOp::JumpIfNotEq(_reg0, _reg1, ref l)) if label == l => true,
@@ -366,12 +366,12 @@ fn label_is_used<'sc>(buf: &[Op<'sc>], label: &Label) -> bool {
 }
 
 #[derive(Default, Clone, Debug)]
-pub struct DataSection<'sc> {
+pub struct DataSection {
     /// the data to be put in the data section of the asm
-    pub value_pairs: Vec<Data<'sc>>,
+    pub value_pairs: Vec<Data>,
 }
 
-impl<'sc> DataSection<'sc> {
+impl DataSection {
     /// Given a [DataId], calculate the offset _from the beginning of the data section_ to the data
     /// in bytes.
     pub(crate) fn offset_to_id(&self, id: &DataId) -> usize {
@@ -392,7 +392,7 @@ impl<'sc> DataSection<'sc> {
     }
 
     /// Calculates the return type of the data held at a specific [DataId].
-    pub(crate) fn type_of_data(&self, id: &DataId) -> Option<ResolvedType<'sc>> {
+    pub(crate) fn type_of_data(&self, id: &DataId) -> Option<ResolvedType> {
         self.value_pairs.get(id.0 as usize).map(|x| x.as_type())
     }
 
@@ -410,7 +410,7 @@ impl<'sc> DataSection<'sc> {
     /// Given any data in the form of a [Literal] (using this type mainly because it includes type
     /// information and debug spans), insert it into the data section and return its offset as a
     /// [DataId].
-    pub(crate) fn insert_data_value(&mut self, data: &Literal<'sc>) -> DataId {
+    pub(crate) fn insert_data_value(&mut self, data: &Literal) -> DataId {
         // if there is an identical data value, use the same id
         match self.value_pairs.iter().position(|x| x == data) {
             Some(num) => DataId(num as u32),
@@ -423,7 +423,7 @@ impl<'sc> DataSection<'sc> {
     }
 }
 
-impl fmt::Display for DataSection<'_> {
+impl fmt::Display for DataSection {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         let mut data_buf = String::new();
         for (ix, data) in self.value_pairs.iter().enumerate() {
@@ -433,7 +433,7 @@ impl fmt::Display for DataSection<'_> {
                 Literal::U32(num) => format!(".u32 {:#04x}", num),
                 Literal::U64(num) => format!(".u64 {:#04x}", num),
                 Literal::Boolean(b) => format!(".bool {}", if *b { "0x01" } else { "0x00" }),
-                Literal::String(st) => format!(".str \"{}\"", st),
+                Literal::String(st) => format!(".str \"{}\"", st.as_str()),
                 Literal::Byte(b) => format!(".byte {:#08b}", b),
                 Literal::B256(b) => format!(
                     ".b256 0x{}",
@@ -451,7 +451,7 @@ impl fmt::Display for DataSection<'_> {
     }
 }
 
-impl fmt::Display for HllAsmSet<'_> {
+impl fmt::Display for HllAsmSet {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
             HllAsmSet::ScriptMain {
@@ -472,7 +472,7 @@ impl fmt::Display for HllAsmSet<'_> {
     }
 }
 
-impl fmt::Display for JumpOptimizedAsmSet<'_> {
+impl fmt::Display for JumpOptimizedAsmSet {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
             JumpOptimizedAsmSet::ScriptMain {
@@ -493,7 +493,7 @@ impl fmt::Display for JumpOptimizedAsmSet<'_> {
     }
 }
 
-impl<'sc> fmt::Display for RegisterAllocatedAsmSet<'sc> {
+impl fmt::Display for RegisterAllocatedAsmSet {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
             RegisterAllocatedAsmSet::ScriptMain {
@@ -520,7 +520,7 @@ impl<'sc> fmt::Display for RegisterAllocatedAsmSet<'sc> {
     }
 }
 
-impl<'sc> fmt::Display for FinalizedAsm<'sc> {
+impl fmt::Display for FinalizedAsm {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
             FinalizedAsm::ScriptMain {
@@ -541,7 +541,7 @@ impl<'sc> fmt::Display for FinalizedAsm<'sc> {
     }
 }
 
-impl fmt::Display for AbstractInstructionSet<'_> {
+impl fmt::Display for AbstractInstructionSet {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         write!(
             f,
@@ -555,7 +555,7 @@ impl fmt::Display for AbstractInstructionSet<'_> {
     }
 }
 
-impl<'sc> fmt::Display for InstructionSet<'sc> {
+impl fmt::Display for InstructionSet {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         write!(
             f,
@@ -570,9 +570,9 @@ impl<'sc> fmt::Display for InstructionSet<'sc> {
 }
 
 #[derive(Default, Clone, Debug)]
-pub(crate) struct AsmNamespace<'sc> {
-    data_section: DataSection<'sc>,
-    variables: HashMap<Ident<'sc>, VirtualRegister>,
+pub(crate) struct AsmNamespace {
+    data_section: DataSection,
+    variables: HashMap<Ident, VirtualRegister>,
 }
 
 /// An address which refers to a value in the data section of the asm.
@@ -585,24 +585,17 @@ impl fmt::Display for DataId {
     }
 }
 
-impl<'sc> AsmNamespace<'sc> {
-    pub(crate) fn insert_variable(
-        &mut self,
-        var_name: Ident<'sc>,
-        register_location: VirtualRegister,
-    ) {
+impl AsmNamespace {
+    pub(crate) fn insert_variable(&mut self, var_name: Ident, register_location: VirtualRegister) {
         self.variables.insert(var_name, register_location);
     }
-    pub(crate) fn insert_data_value(&mut self, data: &Data<'sc>) -> DataId {
+    pub(crate) fn insert_data_value(&mut self, data: &Data) -> DataId {
         self.data_section.insert_data_value(data)
     }
     /// Finds the register which contains variable `var_name`
     /// The `get` is unwrapped, because invalid variable expressions are
     /// checked for in the type checking stage.
-    pub(crate) fn look_up_variable(
-        &self,
-        var_name: &Ident<'sc>,
-    ) -> CompileResult<'sc, &VirtualRegister> {
+    pub(crate) fn look_up_variable(&self, var_name: &Ident) -> CompileResult<&VirtualRegister> {
         match self.variables.get(var_name) {
             Some(o) => ok(o, vec![], vec![]),
             None => err(
@@ -610,17 +603,17 @@ impl<'sc> AsmNamespace<'sc> {
                 vec![CompileError::Internal(
                     "Unknown variable in assembly generation. This should have been an error \
                      during type checking.",
-                    var_name.span.clone(),
+                    var_name.span().clone(),
                 )],
             ),
         }
     }
 }
 
-pub(crate) fn compile_ast_to_asm<'sc>(
-    ast: TypedParseTree<'sc>,
+pub(crate) fn compile_ast_to_asm(
+    ast: TypedParseTree,
     build_config: &BuildConfig,
-) -> CompileResult<'sc, FinalizedAsm<'sc>> {
+) -> CompileResult<FinalizedAsm> {
     let mut register_sequencer = RegisterSequencer::new();
     let mut warnings = vec![];
     let mut errors = vec![];
@@ -789,8 +782,8 @@ pub(crate) fn compile_ast_to_asm<'sc>(
     ok(finalized_asm, warnings, errors)
 }
 
-impl<'sc> HllAsmSet<'sc> {
-    pub(crate) fn remove_unnecessary_jumps(self) -> JumpOptimizedAsmSet<'sc> {
+impl HllAsmSet {
+    pub(crate) fn remove_unnecessary_jumps(self) -> JumpOptimizedAsmSet {
         match self {
             HllAsmSet::ScriptMain {
                 data_section,
@@ -818,8 +811,8 @@ impl<'sc> HllAsmSet<'sc> {
     }
 }
 
-impl<'sc> JumpOptimizedAsmSet<'sc> {
-    fn allocate_registers(self, namespace: &AsmNamespace) -> RegisterAllocatedAsmSet<'sc> {
+impl JumpOptimizedAsmSet {
+    fn allocate_registers(self, namespace: &AsmNamespace) -> RegisterAllocatedAsmSet {
         match self {
             JumpOptimizedAsmSet::Library => RegisterAllocatedAsmSet::Library,
             JumpOptimizedAsmSet::ScriptMain {
@@ -860,42 +853,42 @@ impl<'sc> JumpOptimizedAsmSet<'sc> {
 }
 
 /// Represents an ASM set which has had jump labels and jumps optimized
-pub enum JumpOptimizedAsmSet<'sc> {
+pub enum JumpOptimizedAsmSet {
     ContractAbi {
-        data_section: DataSection<'sc>,
-        program_section: AbstractInstructionSet<'sc>,
+        data_section: DataSection,
+        program_section: AbstractInstructionSet,
     },
     ScriptMain {
-        data_section: DataSection<'sc>,
-        program_section: AbstractInstructionSet<'sc>,
+        data_section: DataSection,
+        program_section: AbstractInstructionSet,
     },
     PredicateMain {
-        data_section: DataSection<'sc>,
-        program_section: AbstractInstructionSet<'sc>,
+        data_section: DataSection,
+        program_section: AbstractInstructionSet,
     },
     // Libraries do not generate any asm.
     Library,
 }
 /// Represents an ASM set which has had registers allocated
-pub enum RegisterAllocatedAsmSet<'sc> {
+pub enum RegisterAllocatedAsmSet {
     ContractAbi {
-        data_section: DataSection<'sc>,
-        program_section: InstructionSet<'sc>,
+        data_section: DataSection,
+        program_section: InstructionSet,
     },
     ScriptMain {
-        data_section: DataSection<'sc>,
-        program_section: InstructionSet<'sc>,
+        data_section: DataSection,
+        program_section: InstructionSet,
     },
     PredicateMain {
-        data_section: DataSection<'sc>,
-        program_section: InstructionSet<'sc>,
+        data_section: DataSection,
+        program_section: InstructionSet,
     },
     // Libraries do not generate any asm.
     Library,
 }
 
-impl<'sc> RegisterAllocatedAsmSet<'sc> {
-    fn optimize(self) -> FinalizedAsm<'sc> {
+impl RegisterAllocatedAsmSet {
+    fn optimize(self) -> FinalizedAsm {
         // TODO implement this -- noop for now
         match self {
             RegisterAllocatedAsmSet::Library => FinalizedAsm::Library,
@@ -957,20 +950,20 @@ impl<'sc> RegisterAllocatedAsmSet<'sc> {
     }
 }
 
-pub(crate) enum NodeAsmResult<'sc> {
-    JustAsm(Vec<Op<'sc>>),
-    ReturnStatement { asm: Vec<Op<'sc>> },
+pub(crate) enum NodeAsmResult {
+    JustAsm(Vec<Op>),
+    ReturnStatement { asm: Vec<Op> },
 }
 
 /// The tuple being returned here contains the opcodes of the code block and,
 /// optionally, a return register in case this node was a return statement
-fn convert_node_to_asm<'sc>(
-    node: &TypedAstNode<'sc>,
-    namespace: &mut AsmNamespace<'sc>,
+fn convert_node_to_asm(
+    node: &TypedAstNode,
+    namespace: &mut AsmNamespace,
     register_sequencer: &mut RegisterSequencer,
     // Where to put the return value of this node, if it is needed.
     return_register: Option<&VirtualRegister>,
-) -> CompileResult<'sc, NodeAsmResult<'sc>> {
+) -> CompileResult<NodeAsmResult> {
     let mut warnings = vec![];
     let mut errors = vec![];
     match &node.content {
@@ -1078,7 +1071,7 @@ fn convert_node_to_asm<'sc>(
 /// 3    LW $ds $is               1 (where 1 is in words and $is is a byte address to base off of)
 /// -    ADD $ds $ds $is
 /// 4    .program_start:
-fn build_preamble(register_sequencer: &mut RegisterSequencer) -> [Op<'static>; 6] {
+fn build_preamble(register_sequencer: &mut RegisterSequencer) -> [Op; 6] {
     let label = register_sequencer.get_label();
     [
         // word 1
@@ -1118,11 +1111,11 @@ fn build_preamble(register_sequencer: &mut RegisterSequencer) -> [Op<'static>; 6
 /// Builds the contract switch statement, or function selector, which takes the selector
 /// stored in the call frame (see https://github.com/FuelLabs/sway/issues/97#issuecomment-870150684
 /// for an explanation of its location)
-fn build_contract_abi_switch<'sc>(
+fn build_contract_abi_switch(
     register_sequencer: &mut RegisterSequencer,
-    namespace: &mut AsmNamespace<'sc>,
+    namespace: &mut AsmNamespace,
     selectors_and_labels: Vec<([u8; 4], Label)>,
-) -> Vec<Op<'sc>> {
+) -> Vec<Op> {
     let input_selector_register = register_sequencer.next();
     let mut asm_buf = vec![Op {
         opcode: Either::Right(OrganizationalOp::Comment),
@@ -1194,13 +1187,13 @@ fn build_contract_abi_switch<'sc>(
     asm_buf
 }
 
-fn add_all_constant_decls<'sc>(
-    namespace: &mut AsmNamespace<'sc>,
+fn add_all_constant_decls(
+    namespace: &mut AsmNamespace,
     register_sequencer: &mut RegisterSequencer,
-    asm_buf: &mut Vec<Op<'sc>>,
-    declarations: &[TypedDeclaration<'sc>],
-    ast_namespace: &Namespace<'sc>,
-) -> CompileResult<'sc, ()> {
+    asm_buf: &mut Vec<Op>,
+    declarations: &[TypedDeclaration],
+    ast_namespace: &Namespace,
+) -> CompileResult<()> {
     let mut warnings = vec![];
     let mut errors = vec![];
     check!(
@@ -1218,12 +1211,12 @@ fn add_all_constant_decls<'sc>(
     ok((), warnings, errors)
 }
 
-fn add_global_constant_decls<'sc>(
-    namespace: &mut AsmNamespace<'sc>,
+fn add_global_constant_decls(
+    namespace: &mut AsmNamespace,
     register_sequencer: &mut RegisterSequencer,
-    asm_buf: &mut Vec<Op<'sc>>,
-    declarations: &[TypedDeclaration<'sc>],
-) -> CompileResult<'sc, ()> {
+    asm_buf: &mut Vec<Op>,
+    declarations: &[TypedDeclaration],
+) -> CompileResult<()> {
     let mut warnings = vec![];
     let mut errors = vec![];
     for declaration in declarations {
@@ -1240,12 +1233,12 @@ fn add_global_constant_decls<'sc>(
     ok((), warnings, errors)
 }
 
-fn add_module_constant_decls<'sc>(
-    namespace: &mut AsmNamespace<'sc>,
+fn add_module_constant_decls(
+    namespace: &mut AsmNamespace,
     register_sequencer: &mut RegisterSequencer,
-    asm_buf: &mut Vec<Op<'sc>>,
-    ast_namespace: &Namespace<'sc>,
-) -> CompileResult<'sc, ()> {
+    asm_buf: &mut Vec<Op>,
+    ast_namespace: &Namespace,
+) -> CompileResult<()> {
     let mut warnings = vec![];
     let mut errors = vec![];
 
@@ -1279,16 +1272,16 @@ fn add_module_constant_decls<'sc>(
 /// The function selector value and corresponding label.
 type JumpDestination = Vec<([u8; 4], Label)>;
 /// A vector of opcodes representing the body of a contract ABI function.
-type AbiFunctionOpcodeBuffer<'sc> = Vec<Op<'sc>>;
+type AbiFunctionOpcodeBuffer = Vec<Op>;
 /// The function selector information and compiled body of a contract ABI function.
-type SerializedAbiFunction<'sc> = (JumpDestination, AbiFunctionOpcodeBuffer<'sc>);
+type SerializedAbiFunction = (JumpDestination, AbiFunctionOpcodeBuffer);
 
 /// Given a contract's abi entries, compile them to jump destinations and an opcode buffer.
-fn compile_contract_to_selectors<'sc>(
-    abi_entries: Vec<TypedFunctionDeclaration<'sc>>,
-    namespace: &mut AsmNamespace<'sc>,
+fn compile_contract_to_selectors(
+    abi_entries: Vec<TypedFunctionDeclaration>,
+    namespace: &mut AsmNamespace,
     register_sequencer: &mut RegisterSequencer,
-) -> CompileResult<'sc, SerializedAbiFunction<'sc>> {
+) -> CompileResult<SerializedAbiFunction> {
     let mut warnings = vec![];
     let mut errors = vec![];
     // for every ABI function, we need:
@@ -1349,7 +1342,7 @@ fn compile_contract_to_selectors<'sc>(
     ok((selectors_labels_buf, asm_buf), warnings, errors)
 }
 /// Given a register, load the user-provided argument into it
-fn load_user_argument<'sc>(return_register: VirtualRegister) -> Op<'sc> {
+fn load_user_argument(return_register: VirtualRegister) -> Op {
     Op {
         opcode: Either::Left(VirtualOp::LW(
             return_register,
@@ -1362,7 +1355,7 @@ fn load_user_argument<'sc>(return_register: VirtualRegister) -> Op<'sc> {
     }
 }
 /// Given a register, load the current value of $cgas into it
-fn load_cgas<'sc>(return_register: VirtualRegister) -> Op<'sc> {
+fn load_cgas(return_register: VirtualRegister) -> Op {
     Op {
         opcode: Either::Left(VirtualOp::LW(
             return_register,
@@ -1374,7 +1367,7 @@ fn load_cgas<'sc>(return_register: VirtualRegister) -> Op<'sc> {
     }
 }
 /// Given a register, load the current value of $bal into it
-fn load_bal<'sc>(return_register: VirtualRegister) -> Op<'sc> {
+fn load_bal(return_register: VirtualRegister) -> Op {
     Op {
         opcode: Either::Left(VirtualOp::LW(
             return_register,
@@ -1386,7 +1379,7 @@ fn load_bal<'sc>(return_register: VirtualRegister) -> Op<'sc> {
     }
 }
 /// Given a register, load a pointer to the current coin color into it
-fn load_coin_color<'sc>(return_register: VirtualRegister) -> Op<'sc> {
+fn load_coin_color(return_register: VirtualRegister) -> Op {
     Op {
         opcode: Either::Left(VirtualOp::LW(
             return_register,
@@ -1400,12 +1393,12 @@ fn load_coin_color<'sc>(return_register: VirtualRegister) -> Op<'sc> {
 
 /// Given a [TypedFunctionDeclaration] and a `return_register`, return
 /// the return value of the function using either a `RET` or a `RETD` opcode.
-fn ret_or_retd_value<'sc>(
-    func: &TypedFunctionDeclaration<'sc>,
+fn ret_or_retd_value(
+    func: &TypedFunctionDeclaration,
     return_register: VirtualRegister,
     register_sequencer: &mut RegisterSequencer,
-    namespace: &mut AsmNamespace<'sc>,
-) -> CompileResult<'sc, Vec<Op<'sc>>> {
+    namespace: &mut AsmNamespace,
+) -> CompileResult<Vec<Op>> {
     let mut errors = vec![];
     let warnings = vec![];
     let mut asm_buf = vec![];
@@ -1417,7 +1410,7 @@ fn ret_or_retd_value<'sc>(
         }
     };
 
-    if main_func_ret_ty == TypeInfo::Unit {
+    if main_func_ret_ty.is_unit() {
         // unit returns should always be zero, although because they can be
         // omitted from functions, the register is sometimes uninitialized.
         // Manually return zero in this case.
@@ -1427,14 +1420,14 @@ fn ret_or_retd_value<'sc>(
                     ConstantRegister::Zero,
                 ))),
                 owning_span: Some(func.return_type_span.clone()),
-                comment: format!("fn {} returns unit", func.name.primary_name),
+                comment: format!("fn {} returns unit", func.name.as_str()),
             }],
             warnings,
             errors,
         );
     }
     let span = crate::Span {
-        span: pest::Span::new("TODO(static span)", 0, 0).unwrap(),
+        span: pest::Span::new("TODO(static span)".into(), 0, 0).unwrap(),
         path: None,
     };
 
@@ -1445,7 +1438,7 @@ fn ret_or_retd_value<'sc>(
         asm_buf.push(Op {
             owning_span: None,
             opcode: Either::Left(VirtualOp::RET(return_register)),
-            comment: format!("{} fn return value", func.name.primary_name),
+            comment: format!("{} fn return value", func.name.as_str()),
         });
     } else {
         // if the type is larger than one word, then we use RETD to return data
@@ -1463,7 +1456,7 @@ fn ret_or_retd_value<'sc>(
         asm_buf.push(Op {
             owning_span: None,
             opcode: Either::Left(VirtualOp::RETD(return_register, rb_register)),
-            comment: format!("{} fn return value", func.name.primary_name),
+            comment: format!("{} fn return value", func.name.as_str()),
         });
     }
     ok(asm_buf, warnings, errors)
