@@ -19,8 +19,8 @@ use crate::{
 use petgraph::algo::has_path_connecting;
 use petgraph::prelude::NodeIndex;
 
-impl<'sc> ControlFlowGraph<'sc> {
-    pub(crate) fn find_dead_code(&self) -> Vec<CompileWarning<'sc>> {
+impl ControlFlowGraph {
+    pub(crate) fn find_dead_code(&self) -> Vec<CompileWarning> {
         // dead code is code that has no path to the entry point
         let mut dead_nodes = vec![];
         for destination in self.graph.node_indices() {
@@ -93,11 +93,11 @@ impl<'sc> ControlFlowGraph<'sc> {
     }
 
     pub(crate) fn append_to_dead_code_graph(
-        ast: &TypedParseTree<'sc>,
-        tree_type: &TreeType<'sc>,
-        graph: &mut ControlFlowGraph<'sc>,
+        ast: &TypedParseTree,
+        tree_type: &TreeType,
+        graph: &mut ControlFlowGraph,
         // the `Result` return is just to handle `Unimplemented` errors
-    ) -> Result<(), CompileError<'sc>> {
+    ) -> Result<(), CompileError> {
         // do a depth first traversal and cover individual inner ast nodes
         let mut leaves = vec![];
         let exit_node = Some(graph.add_node(("Program exit".to_string()).into()));
@@ -126,7 +126,7 @@ impl<'sc> ControlFlowGraph<'sc> {
                                         ),
                                     ),
                                 ..
-                            }) => name.primary_name == "main",
+                            }) => name.as_str() == "main",
                             _ => false,
                         })
                         .unwrap(),
@@ -179,13 +179,13 @@ impl<'sc> ControlFlowGraph<'sc> {
         Ok(())
     }
 }
-fn connect_node<'sc>(
-    node: &TypedAstNode<'sc>,
-    graph: &mut ControlFlowGraph<'sc>,
+fn connect_node(
+    node: &TypedAstNode,
+    graph: &mut ControlFlowGraph,
     leaves: &[NodeIndex],
     exit_node: Option<NodeIndex>,
-    tree_type: &TreeType<'sc>,
-) -> Result<(Vec<NodeIndex>, Option<NodeIndex>), CompileError<'sc>> {
+    tree_type: &TreeType,
+) -> Result<(Vec<NodeIndex>, Option<NodeIndex>), CompileError> {
     //    let mut graph = graph.clone();
     let span = node.span.clone();
     Ok(match &node.content {
@@ -313,15 +313,15 @@ fn connect_node<'sc>(
     })
 }
 
-fn connect_declaration<'sc>(
-    decl: &TypedDeclaration<'sc>,
-    graph: &mut ControlFlowGraph<'sc>,
+fn connect_declaration(
+    decl: &TypedDeclaration,
+    graph: &mut ControlFlowGraph,
     entry_node: NodeIndex,
-    span: Span<'sc>,
+    span: Span,
     exit_node: Option<NodeIndex>,
-    tree_type: &TreeType<'sc>,
+    tree_type: &TreeType,
     leaves: &[NodeIndex],
-) -> Result<Vec<NodeIndex>, CompileError<'sc>> {
+) -> Result<Vec<NodeIndex>, CompileError> {
     use TypedDeclaration::*;
     match decl {
         VariableDeclaration(TypedVariableDeclaration { body, .. }) => connect_expression(
@@ -380,11 +380,11 @@ fn connect_declaration<'sc>(
 
 /// Connect each individual struct field, and when that field is accessed in a subfield expression,
 /// connect that field.
-fn connect_struct_declaration<'sc>(
-    struct_decl: &TypedStructDeclaration<'sc>,
-    graph: &mut ControlFlowGraph<'sc>,
+fn connect_struct_declaration(
+    struct_decl: &TypedStructDeclaration,
+    graph: &mut ControlFlowGraph,
     entry_node: NodeIndex,
-    tree_type: &TreeType<'sc>,
+    tree_type: &TreeType,
 ) {
     let TypedStructDeclaration {
         name,
@@ -413,7 +413,7 @@ fn connect_struct_declaration<'sc>(
     // of the field names
     graph
         .namespace
-        .insert_struct(name.primary_name.to_string(), entry_node, field_nodes);
+        .insert_struct(name.as_str().to_string(), entry_node, field_nodes);
 }
 
 /// Implementations of traits are top-level things that are not conditional, so
@@ -421,13 +421,13 @@ fn connect_struct_declaration<'sc>(
 /// that the declaration was indeed at some point implemented.
 /// Additionally, we insert the trait's methods into the method namespace in order to
 /// track which exact methods are dead code.
-fn connect_impl_trait<'sc>(
-    trait_name: &CallPath<'sc>,
-    graph: &mut ControlFlowGraph<'sc>,
-    methods: &[TypedFunctionDeclaration<'sc>],
+fn connect_impl_trait(
+    trait_name: &CallPath,
+    graph: &mut ControlFlowGraph,
+    methods: &[TypedFunctionDeclaration],
     entry_node: NodeIndex,
-    tree_type: &TreeType<'sc>,
-) -> Result<(), CompileError<'sc>> {
+    tree_type: &TreeType,
+) -> Result<(), CompileError> {
     let graph_c = graph.clone();
     let trait_decl_node = graph_c.namespace.find_trait(trait_name);
     match trait_decl_node {
@@ -483,9 +483,9 @@ fn connect_impl_trait<'sc>(
 ///
 /// The trait node itself has already been added (as `entry_node`), so we just need to insert that
 /// node index into the namespace for the trait.
-fn connect_trait_declaration<'sc>(
-    decl: &TypedTraitDeclaration<'sc>,
-    graph: &mut ControlFlowGraph<'sc>,
+fn connect_trait_declaration(
+    decl: &TypedTraitDeclaration,
+    graph: &mut ControlFlowGraph,
     entry_node: NodeIndex,
 ) {
     graph.namespace.add_trait(
@@ -498,9 +498,9 @@ fn connect_trait_declaration<'sc>(
 }
 
 /// See [connect_trait_declaration] for implementation details.
-fn connect_abi_declaration<'sc>(
-    decl: &TypedAbiDeclaration<'sc>,
-    graph: &mut ControlFlowGraph<'sc>,
+fn connect_abi_declaration(
+    decl: &TypedAbiDeclaration,
+    graph: &mut ControlFlowGraph,
     entry_node: NodeIndex,
 ) {
     graph.namespace.add_trait(
@@ -514,9 +514,9 @@ fn connect_abi_declaration<'sc>(
 /// For an enum declaration, we want to make a declaration node for every individual enum
 /// variant. When a variant is constructed, we can point an edge at that variant. This way,
 /// we can see clearly, and thusly warn, when individual variants are not ever constructed.
-fn connect_enum_declaration<'sc>(
-    enum_decl: &TypedEnumDeclaration<'sc>,
-    graph: &mut ControlFlowGraph<'sc>,
+fn connect_enum_declaration(
+    enum_decl: &TypedEnumDeclaration,
+    graph: &mut ControlFlowGraph,
     entry_node: NodeIndex,
 ) {
     // keep a mapping of each variant
@@ -536,15 +536,15 @@ fn connect_enum_declaration<'sc>(
 /// When connecting a function declaration, we are inserting a new root node into the graph that
 /// has no entry points, since it is just a declaration.
 /// When something eventually calls it, it gets connected to the declaration.
-fn connect_typed_fn_decl<'sc>(
-    fn_decl: &TypedFunctionDeclaration<'sc>,
-    graph: &mut ControlFlowGraph<'sc>,
+fn connect_typed_fn_decl(
+    fn_decl: &TypedFunctionDeclaration,
+    graph: &mut ControlFlowGraph,
     entry_node: NodeIndex,
-    span: Span<'sc>,
+    span: Span,
     exit_node: Option<NodeIndex>,
-    tree_type: &TreeType<'sc>,
-) -> Result<(), CompileError<'sc>> {
-    let fn_exit_node = graph.add_node(format!("\"{}\" fn exit", fn_decl.name.primary_name).into());
+    tree_type: &TreeType,
+) -> Result<(), CompileError> {
+    let fn_exit_node = graph.add_node(format!("\"{}\" fn exit", fn_decl.name.as_str()).into());
     let (_exit_nodes, _exit_node) = depth_first_insertion_code_block(
         &fn_decl.body,
         graph,
@@ -558,7 +558,8 @@ fn connect_typed_fn_decl<'sc>(
 
     // not sure how correct it is to default to Unit here...
     // I think types should all be resolved by now.
-    let ty = resolve_type(fn_decl.return_type, &span).unwrap_or(TypeInfo::Unit);
+    let ty =
+        resolve_type(fn_decl.return_type, &span).unwrap_or_else(|_| TypeInfo::Tuple(Vec::new()));
 
     let namespace_entry = FunctionNamespaceEntry {
         entry_point: entry_node,
@@ -572,13 +573,13 @@ fn connect_typed_fn_decl<'sc>(
     Ok(())
 }
 
-fn depth_first_insertion_code_block<'sc>(
-    node_content: &TypedCodeBlock<'sc>,
-    graph: &mut ControlFlowGraph<'sc>,
+fn depth_first_insertion_code_block(
+    node_content: &TypedCodeBlock,
+    graph: &mut ControlFlowGraph,
     leaves: &[NodeIndex],
     exit_node: Option<NodeIndex>,
-    tree_type: &TreeType<'sc>,
-) -> Result<(Vec<NodeIndex>, Option<NodeIndex>), CompileError<'sc>> {
+    tree_type: &TreeType,
+) -> Result<(Vec<NodeIndex>, Option<NodeIndex>), CompileError> {
     let mut leaves = leaves.to_vec();
     let mut exit_node = exit_node;
     for node in node_content.contents.iter() {
@@ -591,15 +592,15 @@ fn depth_first_insertion_code_block<'sc>(
 
 /// connects any inner parts of an expression to the graph
 /// note the main expression node has already been inserted
-fn connect_expression<'sc>(
-    expr_variant: &TypedExpressionVariant<'sc>,
-    graph: &mut ControlFlowGraph<'sc>,
+fn connect_expression(
+    expr_variant: &TypedExpressionVariant,
+    graph: &mut ControlFlowGraph,
     leaves: &[NodeIndex],
     exit_node: Option<NodeIndex>,
     label: &'static str,
-    tree_type: &TreeType<'sc>,
-    expression_span: Span<'sc>,
-) -> Result<Vec<NodeIndex>, CompileError<'sc>> {
+    tree_type: &TreeType,
+    expression_span: Span,
+) -> Result<Vec<NodeIndex>, CompileError> {
     use TypedExpressionVariant::*;
     match expr_variant {
         FunctionApplication {
@@ -620,13 +621,11 @@ fn connect_expression<'sc>(
                 )
                 .unwrap_or_else(|| {
                     let node_idx =
-                        graph.add_node(format!("extern fn {}()", name.suffix.primary_name).into());
+                        graph.add_node(format!("extern fn {}()", name.suffix.as_str()).into());
                     is_external = true;
                     (
                         node_idx,
-                        graph.add_node(
-                            format!("extern fn {} exit", name.suffix.primary_name).into(),
-                        ),
+                        graph.add_node(format!("extern fn {} exit", name.suffix.as_str()).into()),
                     )
                 });
             for leaf in leaves {
@@ -776,11 +775,9 @@ fn connect_expression<'sc>(
             struct_name,
             fields,
         } => {
-            let decl = match graph.namespace.find_struct_decl(struct_name.primary_name) {
+            let decl = match graph.namespace.find_struct_decl(struct_name.as_str()) {
                 Some(ix) => *ix,
-                None => {
-                    graph.add_node(format!("External struct  {}", struct_name.primary_name).into())
-                }
+                None => graph.add_node(format!("External struct  {}", struct_name.as_str()).into()),
             };
             let entry = graph.add_node("Struct declaration entry".into());
             let exit = graph.add_node("Struct declaration exit".into());
@@ -819,7 +816,7 @@ fn connect_expression<'sc>(
         } => {
             let resolved_type_of_parent =
                 resolve_type(*resolved_type_of_parent, field_to_access_span)
-                    .unwrap_or(TypeInfo::Unit);
+                    .unwrap_or_else(|_| TypeInfo::Tuple(Vec::new()));
 
             assert!(matches!(resolved_type_of_parent, TypeInfo::Struct { .. }));
             let resolved_type_of_parent = match resolved_type_of_parent {
@@ -856,7 +853,34 @@ fn connect_expression<'sc>(
             }
             Ok(vec![asm_node])
         }
-        Unit => Ok(vec![]),
+        Tuple { fields } => {
+            let entry = graph.add_node("tuple entry".into());
+            let exit = graph.add_node("tuple exit".into());
+            // connect current leaves to the beginning of this expr
+            for leaf in leaves {
+                graph.add_edge(*leaf, entry, label.into());
+            }
+
+            let mut current_leaf = vec![entry];
+            // for every field, connect its expression
+            for value in fields {
+                current_leaf = connect_expression(
+                    &value.expression,
+                    graph,
+                    &current_leaf,
+                    exit_node,
+                    "tuple field instantiation",
+                    tree_type,
+                    value.clone().span,
+                )?;
+            }
+
+            // connect the final field to the exit
+            for leaf in current_leaf {
+                graph.add_edge(leaf, exit, "".into());
+            }
+            Ok(vec![exit])
+        }
         AbiCast { address, .. } => connect_expression(
             &address.expression,
             graph,
@@ -916,19 +940,31 @@ fn connect_expression<'sc>(
             )?;
             Ok(prefix_idx)
         }
+        TupleElemAccess { prefix, .. } => {
+            let prefix_idx = connect_expression(
+                &prefix.expression,
+                graph,
+                leaves,
+                exit_node,
+                "",
+                tree_type,
+                prefix.span.clone(),
+            )?;
+            Ok(prefix_idx)
+        }
         a => {
             println!("Unimplemented: {:?}", a);
-            return Err(CompileError::Unimplemented(
+            Err(CompileError::Unimplemented(
                 "Unimplemented dead code analysis for this.",
                 expression_span,
-            ));
+            ))
         }
     }
 }
 
-fn connect_enum_instantiation<'sc>(
-    enum_decl: &TypedEnumDeclaration<'sc>,
-    variant_name: &Ident<'sc>,
+fn connect_enum_instantiation(
+    enum_decl: &TypedEnumDeclaration,
+    variant_name: &Ident,
     graph: &mut ControlFlowGraph,
     leaves: &[NodeIndex],
 ) -> Vec<NodeIndex> {
@@ -940,7 +976,8 @@ fn connect_enum_instantiation<'sc>(
             let node_idx = graph.add_node(
                 format!(
                     "extern enum {}::{}",
-                    enum_name.primary_name, variant_name.primary_name
+                    enum_name.as_str(),
+                    variant_name.as_str()
                 )
                 .into(),
             );
@@ -967,9 +1004,7 @@ fn connect_enum_instantiation<'sc>(
 /// representing its unreached status. For example, we want to say "this function is never called"
 /// if the node is a function declaration, but "this trait is never used" if it is a trait
 /// declaration.
-fn construct_dead_code_warning_from_node<'sc>(
-    node: &TypedAstNode<'sc>,
-) -> Option<CompileWarning<'sc>> {
+fn construct_dead_code_warning_from_node(node: &TypedAstNode) -> Option<CompileWarning> {
     Some(match node {
         // if this is a function, struct, or trait declaration that is never called, then it is dead
         // code.
@@ -998,7 +1033,7 @@ fn construct_dead_code_warning_from_node<'sc>(
                 )),
             ..
         } => CompileWarning {
-            span: name.span.clone(),
+            span: name.span().clone(),
             warning_content: Warning::DeadTrait,
         },
         TypedAstNode {
