@@ -15,7 +15,7 @@ mod parser;
 pub mod semantic_analysis;
 mod span;
 mod style;
-pub(crate) mod type_engine;
+pub mod type_engine;
 
 use crate::asm_generation::checks::check_invalid_opcodes;
 pub use crate::parse_tree::*;
@@ -27,6 +27,7 @@ use control_flow_analysis::{ControlFlowGraph, Graph};
 use pest::iterators::Pair;
 use pest::Parser;
 use std::collections::{HashMap, HashSet};
+use std::sync::Arc;
 
 pub use semantic_analysis::TreeType;
 pub use semantic_analysis::TypedParseTree;
@@ -45,61 +46,61 @@ pub use type_engine::TypeInfo;
 /// it can be a library to be imported into one of the aforementioned
 /// program types.
 #[derive(Debug)]
-pub struct HllParseTree<'sc> {
-    pub tree_type: TreeType<'sc>,
-    pub tree: ParseTree<'sc>,
+pub struct HllParseTree {
+    pub tree_type: TreeType,
+    pub tree: ParseTree,
 }
 
 /// Represents some exportable information that results from compiling some
 /// Sway source code.
 #[derive(Debug)]
-pub struct ParseTree<'sc> {
+pub struct ParseTree {
     /// The untyped AST nodes that constitute this tree's root nodes.
-    pub root_nodes: Vec<AstNode<'sc>>,
+    pub root_nodes: Vec<AstNode>,
     /// The [span::Span] of the entire tree.
-    pub span: span::Span<'sc>,
+    pub span: span::Span,
 }
 
 /// A single [AstNode] represents a node in the parse tree. Note that [AstNode]
 /// is a recursive type and can contain other [AstNode], thus populating the tree.
 #[derive(Debug, Clone)]
-pub struct AstNode<'sc> {
+pub struct AstNode {
     /// The content of this ast node, which could be any control flow structure or other
     /// basic organizational component.
-    pub content: AstNodeContent<'sc>,
+    pub content: AstNodeContent,
     /// The [span::Span] representing this entire [AstNode].
-    pub span: span::Span<'sc>,
+    pub span: span::Span,
 }
 
 /// Represents the various structures that constitute a Sway program.
 #[derive(Debug, Clone)]
-pub enum AstNodeContent<'sc> {
+pub enum AstNodeContent {
     /// A statement of the form `use foo::bar;` or `use ::foo::bar;`
-    UseStatement(UseStatement<'sc>),
+    UseStatement(UseStatement),
     /// A statement of the form `return foo;`
-    ReturnStatement(ReturnStatement<'sc>),
+    ReturnStatement(ReturnStatement),
     /// Any type of declaration, of which there are quite a few. See [Declaration] for more details
     /// on the possible variants.
-    Declaration(Declaration<'sc>),
+    Declaration(Declaration),
     /// Any type of expression, of which there are quite a few. See [Expression] for more details.
-    Expression(Expression<'sc>),
+    Expression(Expression),
     /// An implicit return expression is different from a [AstNodeContent::ReturnStatement] because
     /// it is not a control flow item. Therefore it is a different variant.
     ///
     /// An implicit return expression is an [Expression] at the end of a code block which has no
     /// semicolon, denoting that it is the [Expression] to be returned from that block.
-    ImplicitReturnExpression(Expression<'sc>),
+    ImplicitReturnExpression(Expression),
     /// A control flow element which loops continually until some boolean expression evaluates as
     /// `false`.
-    WhileLoop(WhileLoop<'sc>),
+    WhileLoop(WhileLoop),
     /// A statement of the form `dep foo::bar;` which imports/includes another source file.
-    IncludeStatement(IncludeStatement<'sc>),
+    IncludeStatement(IncludeStatement),
 }
 
-impl<'sc> ParseTree<'sc> {
+impl ParseTree {
     /// Create a new, empty, [ParseTree] from a span which represents the source code that it will
     /// cover.
-    pub(crate) fn new(span: span::Span<'sc>) -> Self {
+    pub(crate) fn new(span: span::Span) -> Self {
         ParseTree {
             root_nodes: Vec::new(),
             span,
@@ -107,35 +108,28 @@ impl<'sc> ParseTree<'sc> {
     }
 
     /// Push a new [AstNode] on to the end of a [ParseTree]'s root nodes.
-    pub(crate) fn push(&mut self, new_node: AstNode<'sc>) {
+    pub(crate) fn push(&mut self, new_node: AstNode) {
         self.root_nodes.push(new_node);
     }
 }
 
-/// Given an input `str` and an optional [BuildConfig], parse the input into a [HllParseTree].
-///
-/// Here, the `'sc` lifetime is introduced to the compilation process. It stands for _source code_,
-/// and all references to `'sc` in the compiler refer to the lifetime of the original source input
-/// `str`.
+/// Given an input `Arc<str>` and an optional [BuildConfig], parse the input into a [HllParseTree].
 ///
 /// # Example
 /// ```
 /// # use sway_core::parse;
 /// # fn main() {
 ///     let input = "script; fn main() -> bool { true }";
-///     let result = parse(input, Default::default());
+///     let result = parse(input.into(), Default::default());
 /// # }
 /// ```
 ///
 /// # Panics
 /// Panics if the generated parser from Pest panics.
-pub fn parse<'sc>(
-    input: &'sc str,
-    config: Option<&BuildConfig>,
-) -> CompileResult<'sc, HllParseTree<'sc>> {
+pub fn parse(input: Arc<str>, config: Option<&BuildConfig>) -> CompileResult<HllParseTree> {
     let mut warnings: Vec<CompileWarning> = Vec::new();
     let mut errors: Vec<CompileError> = Vec::new();
-    let mut parsed = match HllParser::parse(Rule::program, input) {
+    let mut parsed = match HllParser::parse(Rule::program, input.clone()) {
         Ok(o) => o,
         Err(e) => {
             return err(
@@ -159,57 +153,57 @@ pub fn parse<'sc>(
     ok(parsed_root, warnings, errors)
 }
 
-/// Represents the result of compiling Sway code via `compile_to_asm`.
+/// Represents the result of compiling Sway code via [compile_to_asm].
 /// Contains the compiled assets or resulting errors, and any warnings generated.
-pub enum CompilationResult<'sc> {
+pub enum CompilationResult {
     Success {
-        asm: FinalizedAsm<'sc>,
-        warnings: Vec<CompileWarning<'sc>>,
+        asm: FinalizedAsm,
+        warnings: Vec<CompileWarning>,
     },
     Library {
-        name: Ident<'sc>,
-        namespace: Box<Namespace<'sc>>,
-        warnings: Vec<CompileWarning<'sc>>,
+        name: Ident,
+        namespace: Box<Namespace>,
+        warnings: Vec<CompileWarning>,
     },
     Failure {
-        warnings: Vec<CompileWarning<'sc>>,
-        errors: Vec<CompileError<'sc>>,
+        warnings: Vec<CompileWarning>,
+        errors: Vec<CompileError>,
     },
 }
 
-pub enum CompileAstResult<'sc> {
+pub enum CompileAstResult {
     Success {
-        parse_tree: Box<TypedParseTree<'sc>>,
-        tree_type: TreeType<'sc>,
-        warnings: Vec<CompileWarning<'sc>>,
+        parse_tree: Box<TypedParseTree>,
+        tree_type: TreeType,
+        warnings: Vec<CompileWarning>,
     },
     Failure {
-        warnings: Vec<CompileWarning<'sc>>,
-        errors: Vec<CompileError<'sc>>,
+        warnings: Vec<CompileWarning>,
+        errors: Vec<CompileError>,
     },
 }
 
-/// Represents the result of compiling Sway code via `compile_to_bytecode`.
-/// Contains the compiled bytecode in byte form, or, resulting errors, and any warnings generated.
-pub enum BytecodeCompilationResult<'sc> {
+/// Represents the result of compiling Sway code via [compile_to_bytecode].
+/// Contains the compiled bytecode in byte form, or resulting errors, and any warnings generated.
+pub enum BytecodeCompilationResult {
     Success {
         bytes: Vec<u8>,
-        warnings: Vec<CompileWarning<'sc>>,
+        warnings: Vec<CompileWarning>,
     },
     Library {
-        warnings: Vec<CompileWarning<'sc>>,
+        warnings: Vec<CompileWarning>,
     },
     Failure {
-        warnings: Vec<CompileWarning<'sc>>,
-        errors: Vec<CompileError<'sc>>,
+        warnings: Vec<CompileWarning>,
+        errors: Vec<CompileError>,
     },
 }
 
 /// If a given [Rule] exists in the input text, return
 /// that string trimmed. Otherwise, return `None`. This is typically used to find keywords.
-pub fn extract_keyword(line: &str, rule: Rule) -> Option<&str> {
-    if let Ok(pair) = HllParser::parse(rule, line) {
-        Some(pair.as_str().trim())
+pub fn extract_keyword(line: &str, rule: Rule) -> Option<String> {
+    if let Ok(pair) = HllParser::parse(rule, Arc::from(line)) {
+        Some(pair.as_str().trim().to_string())
     } else {
         None
     }
@@ -233,9 +227,9 @@ fn get_end(err: &pest::error::Error<Rule>) -> usize {
 
 /// This struct represents the compilation of an internal dependency
 /// defined through an include statement (the `dep` keyword).
-pub(crate) struct InnerDependencyCompileResult<'sc> {
-    name: Ident<'sc>,
-    namespace: Namespace<'sc>,
+pub(crate) struct InnerDependencyCompileResult {
+    name: Ident,
+    namespace: Namespace,
 }
 /// For internal compiler use.
 /// Compiles an included file and returns its control flow and dead code graphs.
@@ -244,17 +238,17 @@ pub(crate) struct InnerDependencyCompileResult<'sc> {
 /// TODO -- there is _so_ much duplicated code and messiness in this file around the
 /// different types of compilation and stuff. After we get to a good state with the MVP,
 /// clean up the types here with the power of hindsight
-pub(crate) fn compile_inner_dependency<'sc>(
-    input: &'sc str,
-    initial_namespace: &Namespace<'sc>,
+pub(crate) fn compile_inner_dependency(
+    input: Arc<str>,
+    initial_namespace: &Namespace,
     build_config: BuildConfig,
-    dead_code_graph: &mut ControlFlowGraph<'sc>,
+    dead_code_graph: &mut ControlFlowGraph,
     dependency_graph: &mut HashMap<String, HashSet<String>>,
-) -> CompileResult<'sc, InnerDependencyCompileResult<'sc>> {
+) -> CompileResult<InnerDependencyCompileResult> {
     let mut warnings = Vec::new();
     let mut errors = Vec::new();
     let parse_tree = check!(
-        parse(input, Some(&build_config)),
+        parse(input.clone(), Some(&build_config)),
         return err(warnings, errors),
         warnings,
         errors
@@ -265,7 +259,7 @@ pub(crate) fn compile_inner_dependency<'sc>(
             errors.push(CompileError::ImportMustBeLibrary {
                 span: span::Span {
                     span: pest::Span::new(input, 0, 0).unwrap(),
-                    path: Some(build_config.clone().path()),
+                    path: Some(build_config.path()),
                 },
             });
             return err(warnings, errors);
@@ -276,7 +270,7 @@ pub(crate) fn compile_inner_dependency<'sc>(
             parse_tree.tree,
             initial_namespace.clone(),
             &parse_tree.tree_type,
-            &build_config.clone(),
+            &build_config,
             dead_code_graph,
             dependency_graph,
         ),
@@ -309,12 +303,12 @@ pub(crate) fn compile_inner_dependency<'sc>(
     )
 }
 
-pub fn compile_to_ast<'sc>(
-    input: &'sc str,
-    initial_namespace: &Namespace<'sc>,
+pub fn compile_to_ast(
+    input: Arc<str>,
+    initial_namespace: &Namespace,
     build_config: &BuildConfig,
     dependency_graph: &mut HashMap<String, HashSet<String>>,
-) -> CompileAstResult<'sc> {
+) -> CompileAstResult {
     let mut warnings = Vec::new();
     let mut errors = Vec::new();
     let parse_tree = check!(
@@ -367,12 +361,12 @@ pub fn compile_to_ast<'sc>(
 
 /// Given input Sway source code, compile to a [CompilationResult] which contains the asm in opcode
 /// form (not raw bytes/bytecode).
-pub fn compile_to_asm<'sc>(
-    input: &'sc str,
-    initial_namespace: &Namespace<'sc>,
+pub fn compile_to_asm(
+    input: Arc<str>,
+    initial_namespace: &Namespace,
     build_config: BuildConfig,
     dependency_graph: &mut HashMap<String, HashSet<String>>,
-) -> CompilationResult<'sc> {
+) -> CompilationResult {
     match compile_to_ast(input, initial_namespace, &build_config, dependency_graph) {
         CompileAstResult::Failure { warnings, errors } => {
             CompilationResult::Failure { warnings, errors }
@@ -408,12 +402,12 @@ pub fn compile_to_asm<'sc>(
 
 /// Given input Sway source code, compile to a [BytecodeCompilationResult] which contains the asm in
 /// bytecode form.
-pub fn compile_to_bytecode<'sc>(
-    input: &'sc str,
-    initial_namespace: &Namespace<'sc>,
+pub fn compile_to_bytecode(
+    input: Arc<str>,
+    initial_namespace: &Namespace,
     build_config: BuildConfig,
     dependency_graph: &mut HashMap<String, HashSet<String>>,
-) -> BytecodeCompilationResult<'sc> {
+) -> BytecodeCompilationResult {
     match compile_to_asm(input, initial_namespace, build_config, dependency_graph) {
         CompilationResult::Success {
             mut asm,
@@ -445,11 +439,11 @@ pub fn compile_to_bytecode<'sc>(
 
 /// Given a [TypedParseTree], which is type-checked Sway source, construct a graph to analyze
 /// control flow and determine if it is valid.
-fn perform_control_flow_analysis<'sc>(
-    tree: &TypedParseTree<'sc>,
-    tree_type: &TreeType<'sc>,
-    dead_code_graph: &mut ControlFlowGraph<'sc>,
-) -> (Vec<CompileWarning<'sc>>, Vec<CompileError<'sc>>) {
+fn perform_control_flow_analysis(
+    tree: &TypedParseTree,
+    tree_type: &TreeType,
+    dead_code_graph: &mut ControlFlowGraph,
+) -> (Vec<CompileWarning>, Vec<CompileError>) {
     match ControlFlowGraph::append_to_dead_code_graph(tree, tree_type, dead_code_graph) {
         Ok(_) => (),
         Err(e) => return (vec![], vec![e]),
@@ -464,10 +458,10 @@ fn perform_control_flow_analysis<'sc>(
 
 /// The basic recursive parser which handles the top-level parsing given the output of the
 /// pest-generated parser.
-fn parse_root_from_pairs<'sc>(
-    input: impl Iterator<Item = Pair<'sc, Rule>>,
+fn parse_root_from_pairs(
+    input: impl Iterator<Item = Pair<Rule>>,
     config: Option<&BuildConfig>,
-) -> CompileResult<'sc, HllParseTree<'sc>> {
+) -> CompileResult<HllParseTree> {
     let path = config.map(|config| config.dir_of_code.clone());
     let mut warnings = Vec::new();
     let mut errors = Vec::new();
@@ -504,13 +498,15 @@ fn parse_root_from_pairs<'sc>(
                         warnings,
                         errors
                     );
-                    parse_tree.push(AstNode {
-                        content: AstNodeContent::UseStatement(stmt),
-                        span: span::Span {
-                            span: pair.as_span(),
-                            path: path.clone(),
-                        },
-                    });
+                    for entry in stmt {
+                        parse_tree.push(AstNode {
+                            content: AstNodeContent::UseStatement(entry.clone()),
+                            span: span::Span {
+                                span: pair.as_span(),
+                                path: path.clone(),
+                            },
+                        });
+                    }
                 }
                 Rule::library_name => {
                     let lib_pair = pair.into_inner().next().unwrap();
@@ -668,7 +664,8 @@ fn test_basic_prog() {
          func_app(my_args, (so_many_args))];
         return 5;
     }
-    "#,
+    "#
+        .into(),
         None,
     );
     let mut warnings: Vec<CompileWarning> = Vec::new();
@@ -684,7 +681,8 @@ fn test_parenthesized() {
             let x = (5 + 6 / (1 + (2 / 1) + 4));
             return;
         }
-    "#,
+    "#
+        .into(),
         None,
     );
     let mut warnings: Vec<CompileWarning> = Vec::new();
@@ -702,7 +700,8 @@ fn test_unary_ordering() {
         let a = true;
         let b = true;
         !a && b;
-    }"#,
+    }"#
+        .into(),
         None,
     );
     let mut warnings: Vec<CompileWarning> = Vec::new();
