@@ -27,76 +27,78 @@ pub(crate) use method_name::MethodName;
 pub(crate) use scrutinee::{Scrutinee, StructScrutineeField};
 pub(crate) use unary_op::UnaryOp;
 
+/// Represents a parsed, but not yet type checked, [Expression](https://en.wikipedia.org/wiki/Expression_(computer_science)).
 #[derive(Debug, Clone)]
-pub enum Expression<'sc> {
+pub enum Expression {
     Literal {
-        value: Literal<'sc>,
-        span: Span<'sc>,
+        value: Literal,
+        span: Span,
     },
     FunctionApplication {
-        name: CallPath<'sc>,
-        arguments: Vec<Expression<'sc>>,
-        type_arguments: Vec<(TypeInfo, Span<'sc>)>,
-        span: Span<'sc>,
+        name: CallPath,
+        arguments: Vec<Expression>,
+        type_arguments: Vec<(TypeInfo, Span)>,
+        span: Span,
     },
     LazyOperator {
         op: LazyOp,
-        lhs: Box<Expression<'sc>>,
-        rhs: Box<Expression<'sc>>,
-        span: Span<'sc>,
+        lhs: Box<Expression>,
+        rhs: Box<Expression>,
+        span: Span,
     },
     VariableExpression {
-        name: Ident<'sc>,
-        span: Span<'sc>,
+        name: Ident,
+        span: Span,
     },
-    Unit {
-        span: Span<'sc>,
+    Tuple {
+        fields: Vec<Expression>,
+        span: Span,
     },
     Array {
-        contents: Vec<Expression<'sc>>,
-        span: Span<'sc>,
+        contents: Vec<Expression>,
+        span: Span,
     },
     MatchExpression {
-        primary_expression: Box<Expression<'sc>>,
-        branches: Vec<MatchBranch<'sc>>,
-        span: Span<'sc>,
+        primary_expression: Box<Expression>,
+        branches: Vec<MatchBranch>,
+        span: Span,
     },
     StructExpression {
-        struct_name: Ident<'sc>,
-        fields: Vec<StructExpressionField<'sc>>,
-        span: Span<'sc>,
+        struct_name: Ident,
+        fields: Vec<StructExpressionField>,
+        span: Span,
     },
     CodeBlock {
-        contents: CodeBlock<'sc>,
-        span: Span<'sc>,
+        contents: CodeBlock,
+        span: Span,
     },
     IfExp {
-        condition: Box<Expression<'sc>>,
-        then: Box<Expression<'sc>>,
-        r#else: Option<Box<Expression<'sc>>>,
-        span: Span<'sc>,
+        condition: Box<Expression>,
+        then: Box<Expression>,
+        r#else: Option<Box<Expression>>,
+        span: Span,
     },
     // separated into other struct for parsing reasons
     AsmExpression {
-        span: Span<'sc>,
-        asm: AsmExpression<'sc>,
+        span: Span,
+        asm: AsmExpression,
     },
     MethodApplication {
-        method_name: MethodName<'sc>,
-        arguments: Vec<Expression<'sc>>,
-        span: Span<'sc>,
+        method_name: MethodName,
+        arguments: Vec<Expression>,
+        span: Span,
     },
-    /// A subfield expression is anything of the form:
+    /// A _subfield expression_ is anything of the form:
     /// ```ignore
     /// <ident>.<ident>
     /// ```
     ///
     SubfieldExpression {
-        prefix: Box<Expression<'sc>>,
-        span: Span<'sc>,
-        field_to_access: Ident<'sc>,
+        prefix: Box<Expression>,
+        span: Span,
+        field_to_access: Ident,
     },
-    /// A [DelineatedPath] is anything of the form:
+    /// A _delineated path_ is anything of the form:
     /// ```ignore
     /// <ident>::<ident>
     /// ```
@@ -118,21 +120,21 @@ pub enum Expression<'sc> {
     /// MyEnum::Variant1
     /// ```
     DelineatedPath {
-        call_path: CallPath<'sc>,
-        args: Vec<Expression<'sc>>,
-        span: Span<'sc>,
+        call_path: CallPath,
+        args: Vec<Expression>,
+        span: Span,
         type_arguments: Vec<TypeInfo>,
     },
     /// A cast of a hash to an ABI for calling a contract.
     AbiCast {
-        abi_name: CallPath<'sc>,
-        address: Box<Expression<'sc>>,
-        span: Span<'sc>,
+        abi_name: CallPath,
+        address: Box<Expression>,
+        span: Span,
     },
     ArrayIndex {
-        prefix: Box<Expression<'sc>>,
-        index: Box<Expression<'sc>>,
-        span: Span<'sc>,
+        prefix: Box<Expression>,
+        index: Box<Expression>,
+        span: Span,
     },
     /// This variant serves as a stand-in for parsing-level match expression desugaring.
     /// Because types cannot be known at parsing-time, a desugared struct or enum gets
@@ -141,33 +143,41 @@ pub enum Expression<'sc> {
     /// expression inside of the delayed resolution has the appropriate struct or enum
     /// type)
     DelayedMatchTypeResolution {
-        variant: DelayedResolutionVariant<'sc>,
-        span: Span<'sc>,
+        variant: DelayedResolutionVariant,
+        span: Span,
     },
 }
 
 #[derive(Debug, Clone)]
-pub enum DelayedResolutionVariant<'sc> {
-    StructField(DelayedStructFieldResolution<'sc>),
-    EnumVariant(DelayedEnumVariantResolution<'sc>),
+pub enum DelayedResolutionVariant {
+    StructField(DelayedStructFieldResolution),
+    EnumVariant(DelayedEnumVariantResolution),
+    TupleVariant(DelayedTupleVariantResolution),
 }
 
 /// During type checking, this gets replaced with struct field access.
 #[derive(Debug, Clone)]
-pub struct DelayedStructFieldResolution<'sc> {
-    pub exp: Box<Expression<'sc>>,
-    pub struct_name: Ident<'sc>,
-    pub field: Ident<'sc>,
+pub struct DelayedStructFieldResolution {
+    pub exp: Box<Expression>,
+    pub struct_name: Ident,
+    pub field: Ident,
 }
 
 /// During type checking, this gets replaced with enum arg access.
 #[derive(Debug, Clone)]
-pub struct DelayedEnumVariantResolution<'sc> {
-    pub exp: Box<Expression<'sc>>,
-    pub call_path: CallPath<'sc>,
+pub struct DelayedEnumVariantResolution {
+    pub exp: Box<Expression>,
+    pub call_path: CallPath,
     pub arg_num: usize,
 }
 
+/// During type checking, this gets replaced with tuple arg access.
+#[derive(Debug, Clone)]
+pub struct DelayedTupleVariantResolution {
+    pub exp: Box<Expression>,
+    pub elem_num: usize,
+}
+
 #[derive(Clone, Debug, PartialEq)]
 pub enum LazyOp {
     And,
@@ -185,26 +195,20 @@ impl LazyOp {
 }
 
 #[derive(Debug, Clone)]
-pub struct StructExpressionField<'sc> {
-    pub(crate) name: Ident<'sc>,
-    pub(crate) value: Expression<'sc>,
-    pub(crate) span: Span<'sc>,
+pub struct StructExpressionField {
+    pub(crate) name: Ident,
+    pub(crate) value: Expression,
+    pub(crate) span: Span,
 }
 
-impl<'sc> Expression<'sc> {
-    pub(crate) fn core_ops_eq(arguments: Vec<Expression<'sc>>, span: Span<'sc>) -> Expression<'sc> {
+impl Expression {
+    pub(crate) fn core_ops_eq(arguments: Vec<Expression>, span: Span) -> Expression {
         Expression::MethodApplication {
             method_name: MethodName::FromType {
                 call_path: CallPath {
                     prefixes: vec![
-                        Ident {
-                            primary_name: "core",
-                            span: span.clone(),
-                        },
-                        Ident {
-                            primary_name: "ops",
-                            span: span.clone(),
-                        },
+                        Ident::new_with_override("core", span.clone()),
+                        Ident::new_with_override("ops", span.clone()),
                     ],
                     suffix: Op {
                         op_variant: OpVariant::Equals,
@@ -220,23 +224,13 @@ impl<'sc> Expression<'sc> {
         }
     }
 
-    pub(crate) fn core_ops(
-        op: Op<'sc>,
-        arguments: Vec<Expression<'sc>>,
-        span: Span<'sc>,
-    ) -> Expression<'sc> {
+    pub(crate) fn core_ops(op: Op, arguments: Vec<Expression>, span: Span) -> Expression {
         Expression::MethodApplication {
             method_name: MethodName::FromType {
                 call_path: CallPath {
                     prefixes: vec![
-                        Ident {
-                            primary_name: "core",
-                            span: span.clone(),
-                        },
-                        Ident {
-                            primary_name: "ops",
-                            span: span.clone(),
-                        },
+                        Ident::new_with_override("core", span.clone()),
+                        Ident::new_with_override("ops", span.clone()),
                     ],
                     suffix: op.to_var_name(),
                 },
@@ -248,14 +242,14 @@ impl<'sc> Expression<'sc> {
         }
     }
 
-    pub(crate) fn span(&self) -> Span<'sc> {
+    pub(crate) fn span(&self) -> Span {
         use Expression::*;
         (match self {
             Literal { span, .. } => span,
             FunctionApplication { span, .. } => span,
             LazyOperator { span, .. } => span,
             VariableExpression { span, .. } => span,
-            Unit { span } => span,
+            Tuple { span, .. } => span,
             Array { span, .. } => span,
             MatchExpression { span, .. } => span,
             StructExpression { span, .. } => span,
@@ -272,9 +266,9 @@ impl<'sc> Expression<'sc> {
         .clone()
     }
     pub(crate) fn parse_from_pair(
-        expr: Pair<'sc, Rule>,
+        expr: Pair<Rule>,
         config: Option<&BuildConfig>,
-    ) -> CompileResult<'sc, Self> {
+    ) -> CompileResult<Self> {
         let path = config.map(|c| c.path());
         let mut warnings = Vec::new();
         let mut errors = Vec::new();
@@ -284,7 +278,8 @@ impl<'sc> Expression<'sc> {
         let first_expr = expr_iter.next().unwrap();
         let first_expr = check!(
             Expression::parse_from_pair_inner(first_expr.clone(), config),
-            Expression::Unit {
+            Expression::Tuple {
+                fields: vec![],
                 span: Span {
                     span: first_expr.as_span(),
                     path: path.clone(),
@@ -297,7 +292,7 @@ impl<'sc> Expression<'sc> {
             vec![Either::Right(first_expr.clone())];
         // sometimes exprs are followed by ops in the same expr
         while let Some(op) = expr_iter.next() {
-            let op_str = op.as_str();
+            let op_str = op.as_str().to_string();
             let op_span = Span {
                 span: op.as_span(),
                 path: path.clone(),
@@ -314,7 +309,8 @@ impl<'sc> Expression<'sc> {
             let next_expr = match expr_iter.next() {
                 Some(o) => check!(
                     Expression::parse_from_pair_inner(o.clone(), config),
-                    Expression::Unit {
+                    Expression::Tuple {
+                        fields: vec![],
                         span: Span {
                             span: o.as_span(),
                             path: path.clone()
@@ -331,7 +327,10 @@ impl<'sc> Expression<'sc> {
                             path: path.clone(),
                         },
                     });
-                    Expression::Unit { span: op_span }
+                    Expression::Tuple {
+                        fields: vec![],
+                        span: op_span,
+                    }
                 }
             };
             // pushing these into a vec in this manner so we can re-associate according to order of
@@ -354,7 +353,8 @@ impl<'sc> Expression<'sc> {
                     path: path.clone(),
                 },
             )
-            .unwrap_or_else(&mut warnings, &mut errors, || Expression::Unit {
+            .unwrap_or_else(&mut warnings, &mut errors, || Expression::Tuple {
+                fields: vec![],
                 span: Span {
                     span: expr_for_debug.as_span(),
                     path: path.clone(),
@@ -365,9 +365,9 @@ impl<'sc> Expression<'sc> {
     }
 
     pub(crate) fn parse_from_pair_inner(
-        expr: Pair<'sc, Rule>,
+        expr: Pair<Rule>,
         config: Option<&BuildConfig>,
-    ) -> CompileResult<'sc, Self> {
+    ) -> CompileResult<Self> {
         let path = config.map(|c| c.path());
         let mut errors = Vec::new();
         let mut warnings = Vec::new();
@@ -378,9 +378,12 @@ impl<'sc> Expression<'sc> {
         #[allow(unused_assignments)]
         let mut maybe_type_args = Vec::new();
         let parsed = match expr.as_rule() {
-            Rule::literal_value => Literal::parse_from_pair(expr.clone(), config)
+            Rule::literal_value => Literal::parse_from_pair(expr, config)
                 .map(|(value, span)| Expression::Literal { value, span })
-                .unwrap_or_else(&mut warnings, &mut errors, || Expression::Unit { span }),
+                .unwrap_or_else(&mut warnings, &mut errors, || Expression::Tuple {
+                    fields: vec![],
+                    span,
+                }),
             Rule::func_app => {
                 let span = Span {
                     span: expr.as_span(),
@@ -408,7 +411,8 @@ impl<'sc> Expression<'sc> {
                 for argument in arguments.into_inner() {
                     let arg = check!(
                         Expression::parse_from_pair(argument.clone(), config),
-                        Expression::Unit {
+                        Expression::Tuple {
+                            fields: vec![],
                             span: Span {
                                 span: argument.as_span(),
                                 path: path.clone()
@@ -452,10 +456,7 @@ impl<'sc> Expression<'sc> {
                         Rule::var_name_ident => {
                             name = Some(check!(
                                 Ident::parse_from_pair(pair, config),
-                                Ident {
-                                    primary_name: "error parsing var name",
-                                    span: span.clone()
-                                },
+                                Ident::new_with_override("error parsing var name", span.clone()),
                                 warnings,
                                 errors
                             ));
@@ -474,7 +475,10 @@ impl<'sc> Expression<'sc> {
                 },
                 Some(array_elems) => check!(
                     parse_array_elems(array_elems, config),
-                    Expression::Unit { span: span.clone() },
+                    Expression::Tuple {
+                        fields: vec![],
+                        span,
+                    },
                     warnings,
                     errors
                 ),
@@ -483,7 +487,10 @@ impl<'sc> Expression<'sc> {
                 let mut expr_iter = expr.into_inner();
                 let primary_expression = check!(
                     Expression::parse_from_pair(expr_iter.next().unwrap(), config),
-                    Expression::Unit { span: span.clone() },
+                    Expression::Tuple {
+                        fields: vec![],
+                        span: span.clone()
+                    },
                     warnings,
                     errors
                 );
@@ -493,7 +500,10 @@ impl<'sc> Expression<'sc> {
                         MatchBranch::parse_from_pair(exp, config),
                         MatchBranch {
                             condition: MatchCondition::CatchAll(CatchAll { span: span.clone() }),
-                            result: Expression::Unit { span: span.clone() },
+                            result: Expression::Tuple {
+                                fields: vec![],
+                                span: span.clone(),
+                            },
                             span: span.clone()
                         },
                         warnings,
@@ -501,13 +511,12 @@ impl<'sc> Expression<'sc> {
                     );
                     branches.push(res);
                 }
-                let exp = check!(
+                check!(
                     desugar_match_expression(primary_expression, branches, span),
                     return err(warnings, errors),
                     warnings,
                     errors
-                );
-                exp
+                )
             }
             Rule::struct_expression => {
                 let mut expr_iter = expr.into_inner();
@@ -533,7 +542,10 @@ impl<'sc> Expression<'sc> {
                     };
                     let value = check!(
                         Expression::parse_from_pair(fields[i + 1].clone(), config),
-                        Expression::Unit { span: span.clone() },
+                        Expression::Tuple {
+                            fields: vec![],
+                            span: span.clone()
+                        },
                         warnings,
                         errors
                     );
@@ -547,9 +559,10 @@ impl<'sc> Expression<'sc> {
                 }
             }
             Rule::parenthesized_expression => {
-                let expr = check!(
+                check!(
                     Expression::parse_from_pair(expr.clone().into_inner().next().unwrap(), config),
-                    Expression::Unit {
+                    Expression::Tuple {
+                        fields: vec![],
                         span: Span {
                             span: expr.as_span(),
                             path,
@@ -557,8 +570,7 @@ impl<'sc> Expression<'sc> {
                     },
                     warnings,
                     errors
-                );
-                expr
+                )
             }
             Rule::code_block => {
                 let whole_block_span = Span {
@@ -590,20 +602,29 @@ impl<'sc> Expression<'sc> {
                 let else_pair = if_exp_pairs.next();
                 let condition = Box::new(check!(
                     Expression::parse_from_pair(condition_pair, config),
-                    Expression::Unit { span: span.clone() },
+                    Expression::Tuple {
+                        fields: vec![],
+                        span: span.clone()
+                    },
                     warnings,
                     errors
                 ));
                 let then = Box::new(check!(
                     Expression::parse_from_pair_inner(then_pair, config),
-                    Expression::Unit { span: span.clone() },
+                    Expression::Tuple {
+                        fields: vec![],
+                        span: span.clone()
+                    },
                     warnings,
                     errors
                 ));
                 let r#else = else_pair.map(|else_pair| {
                     Box::new(check!(
                         Expression::parse_from_pair_inner(else_pair, config),
-                        Expression::Unit { span: span.clone() },
+                        Expression::Tuple {
+                            fields: vec![],
+                            span: span.clone()
+                        },
                         warnings,
                         errors
                     ))
@@ -665,7 +686,8 @@ impl<'sc> Expression<'sc> {
                         for argument in function_arguments {
                             let arg = check!(
                                 Expression::parse_from_pair(argument.clone(), config),
-                                Expression::Unit {
+                                Expression::Tuple {
+                                    fields: vec![],
                                     span: Span {
                                         span: argument.as_span(),
                                         path: path.clone()
@@ -774,7 +796,8 @@ impl<'sc> Expression<'sc> {
                             for argument in arguments.into_inner() {
                                 let arg = check!(
                                     Expression::parse_from_pair(argument.clone(), config),
-                                    Expression::Unit {
+                                    Expression::Tuple {
+                                        fields: vec![],
                                         span: Span {
                                             span: argument.as_span(),
                                             path: path.clone()
@@ -840,12 +863,26 @@ impl<'sc> Expression<'sc> {
                     type_arguments: vec![],
                 }
             }
-            Rule::unit => Expression::Unit {
-                span: Span {
-                    span: expr.as_span(),
-                    path,
-                },
-            },
+            Rule::tuple_expr => {
+                let fields = expr.into_inner().collect::<Vec<_>>();
+                let mut fields_buf = Vec::with_capacity(fields.len());
+                for field in fields {
+                    let value = check!(
+                        Expression::parse_from_pair(field.clone(), config),
+                        Expression::Tuple {
+                            fields: vec![],
+                            span: span.clone()
+                        },
+                        warnings,
+                        errors
+                    );
+                    fields_buf.push(value);
+                }
+                Expression::Tuple {
+                    fields: fields_buf,
+                    span,
+                }
+            }
             Rule::struct_field_access => {
                 let inner = expr.into_inner().next().expect("guaranteed by grammar");
                 assert_eq!(inner.as_rule(), Rule::subfield_path);
@@ -932,7 +969,8 @@ impl<'sc> Expression<'sc> {
                     },
                 ));
                 // construct unit expression for error recovery
-                Expression::Unit {
+                Expression::Tuple {
+                    fields: vec![],
                     span: Span {
                         span: expr.as_span(),
                         path,
@@ -944,10 +982,10 @@ impl<'sc> Expression<'sc> {
     }
 }
 
-fn convert_unary_to_fn_calls<'sc>(
-    item: Pair<'sc, Rule>,
+fn convert_unary_to_fn_calls(
+    item: Pair<Rule>,
     config: Option<&BuildConfig>,
-) -> CompileResult<'sc, Expression<'sc>> {
+) -> CompileResult<Expression> {
     let iter = item.into_inner();
     let mut unary_stack = vec![];
     let mut warnings = vec![];
@@ -990,10 +1028,10 @@ fn convert_unary_to_fn_calls<'sc>(
     ok(expr, warnings, errors)
 }
 
-pub(crate) fn parse_array_index<'sc>(
-    item: Pair<'sc, Rule>,
+pub(crate) fn parse_array_index(
+    item: Pair<Rule>,
     config: Option<&BuildConfig>,
-) -> CompileResult<'sc, Expression<'sc>> {
+) -> CompileResult<Expression> {
     let mut warnings = vec![];
     let mut errors = vec![];
     let path = config.map(|c| c.path());
@@ -1019,7 +1057,7 @@ pub(crate) fn parse_array_index<'sc>(
         prefix: Box::new(prefix),
         index: Box::new(first_index.to_owned()),
         span: Span {
-            span,
+            span: span.clone(),
             path: path.clone(),
         },
     };
@@ -1028,7 +1066,7 @@ pub(crate) fn parse_array_index<'sc>(
             prefix: Box::new(exp),
             index: Box::new(index),
             span: Span {
-                span,
+                span: span.clone(),
                 path: path.clone(),
             },
         };
@@ -1036,10 +1074,10 @@ pub(crate) fn parse_array_index<'sc>(
     ok(exp, warnings, errors)
 }
 
-fn parse_subfield_path<'sc>(
-    item: Pair<'sc, Rule>,
+fn parse_subfield_path(
+    item: Pair<Rule>,
     config: Option<&BuildConfig>,
-) -> CompileResult<'sc, Expression<'sc>> {
+) -> CompileResult<Expression> {
     let warnings = vec![];
     let mut errors = vec![];
     let path = config.map(|c| c.path());
@@ -1062,7 +1100,8 @@ fn parse_subfield_path<'sc>(
                 },
             ));
             // construct unit expression for error recovery
-            let exp = Expression::Unit {
+            let exp = Expression::Tuple {
+                fields: vec![],
                 span: Span {
                     span: item.as_span(),
                     path,
@@ -1076,10 +1115,7 @@ fn parse_subfield_path<'sc>(
 // A call item is parsed as either an `ident` or a parenthesized `expr`. This method's job is to
 // figure out which variant of `call_item` this is and turn it into either a variable expression
 // or parse it as an expression otherwise.
-fn parse_call_item<'sc>(
-    item: Pair<'sc, Rule>,
-    config: Option<&BuildConfig>,
-) -> CompileResult<'sc, Expression<'sc>> {
+fn parse_call_item(item: Pair<Rule>, config: Option<&BuildConfig>) -> CompileResult<Expression> {
     let mut warnings = vec![];
     let mut errors = vec![];
     assert_eq!(item.as_rule(), Rule::call_item);
@@ -1108,10 +1144,7 @@ fn parse_call_item<'sc>(
     ok(exp, warnings, errors)
 }
 
-fn parse_array_elems<'sc>(
-    elems: Pair<'sc, Rule>,
-    config: Option<&BuildConfig>,
-) -> CompileResult<'sc, Expression<'sc>> {
+fn parse_array_elems(elems: Pair<Rule>, config: Option<&BuildConfig>) -> CompileResult<Expression> {
     let mut warnings = Vec::new();
     let mut errors = Vec::new();
 
@@ -1129,7 +1162,8 @@ fn parse_array_elems<'sc>(
             let span = first_elem.as_span();
             let init = Literal::parse_from_pair(first_elem, config)
                 .map(|(value, span)| Expression::Literal { value, span })
-                .unwrap_or_else(&mut warnings, &mut errors, || Expression::Unit {
+                .unwrap_or_else(&mut warnings, &mut errors, || Expression::Tuple {
+                    fields: vec![],
                     span: Span { span, path },
                 });
 
@@ -1152,7 +1186,8 @@ fn parse_array_elems<'sc>(
             let span = first_elem.as_span();
             let first_elem_expr = check!(
                 Expression::parse_from_pair(first_elem, config),
-                Expression::Unit {
+                Expression::Tuple {
+                    fields: vec![],
                     span: Span {
                         span,
                         path: path.clone()
@@ -1165,7 +1200,8 @@ fn parse_array_elems<'sc>(
                 let span = pair.as_span();
                 elems.push(check!(
                     Expression::parse_from_pair(pair, config),
-                    Expression::Unit {
+                    Expression::Tuple {
+                        fields: vec![],
                         span: Span {
                             span,
                             path: path.clone()
@@ -1182,7 +1218,7 @@ fn parse_array_elems<'sc>(
     ok(Expression::Array { contents, span }, warnings, errors)
 }
 
-fn parse_op<'sc>(op: Pair<'sc, Rule>, config: Option<&BuildConfig>) -> CompileResult<'sc, Op<'sc>> {
+fn parse_op(op: Pair<Rule>, config: Option<&BuildConfig>) -> CompileResult<Op> {
     let path = config.map(|c| c.path());
     use OpVariant::*;
     let mut errors = Vec::new();
@@ -1205,7 +1241,7 @@ fn parse_op<'sc>(op: Pair<'sc, Rule>, config: Option<&BuildConfig>) -> CompileRe
         "<=" => LessThanOrEqualTo,
         a => {
             errors.push(CompileError::ExpectedOp {
-                op: a,
+                op: a.to_string(),
                 span: Span {
                     span: op.as_span(),
                     path,
@@ -1228,20 +1264,17 @@ fn parse_op<'sc>(op: Pair<'sc, Rule>, config: Option<&BuildConfig>) -> CompileRe
 }
 
 #[derive(Debug)]
-pub(crate) struct Op<'sc> {
-    pub span: Span<'sc>,
+pub(crate) struct Op {
+    pub span: Span,
     pub op_variant: OpVariant,
 }
 
-impl<'sc> Op<'sc> {
-    pub fn to_var_name(&self) -> Ident<'sc> {
-        Ident {
-            primary_name: self.op_variant.as_str(),
-            span: self.span.clone(),
-            // TODO this should be a method exp not a var name
-        }
+impl Op {
+    pub fn to_var_name(&self) -> Ident {
+        Ident::new_with_override(self.op_variant.as_str(), self.span.clone())
     }
 }
+
 #[derive(Debug)]
 pub enum OpVariant {
     Add,
@@ -1313,10 +1346,10 @@ impl OpVariant {
     }
 }
 
-fn arrange_by_order_of_operations<'sc>(
-    expressions: Vec<Either<Op<'sc>, Expression<'sc>>>,
-    debug_span: Span<'sc>,
-) -> CompileResult<'sc, Expression<'sc>> {
+fn arrange_by_order_of_operations(
+    expressions: Vec<Either<Op, Expression>>,
+    debug_span: Span,
+) -> CompileResult<Expression> {
     let mut errors = Vec::new();
     let warnings = Vec::new();
     let mut expression_stack = Vec::new();
@@ -1416,11 +1449,11 @@ fn arrange_by_order_of_operations<'sc>(
     ok(expression_stack[0].clone(), warnings, errors)
 }
 
-struct MatchedBranch<'sc> {
-    result: Expression<'sc>,
-    match_req_map: Vec<(Expression<'sc>, Expression<'sc>)>,
-    match_impl_map: Vec<(Ident<'sc>, Expression<'sc>)>,
-    branch_span: Span<'sc>,
+struct MatchedBranch {
+    result: Expression,
+    match_req_map: Vec<(Expression, Expression)>,
+    match_impl_map: Vec<(Ident, Expression)>,
+    branch_span: Span,
 }
 
 /// This algorithm desugars match expressions to if statements.
@@ -1467,11 +1500,11 @@ struct MatchedBranch<'sc> {
 ///     2b. Assemble the statements that go inside of the body of the if expression
 ///     2c. Assemble the giant if statement.
 /// 3. Return!
-pub fn desugar_match_expression<'sc>(
-    primary_expression: Expression<'sc>,
-    branches: Vec<MatchBranch<'sc>>,
-    _span: Span<'sc>,
-) -> CompileResult<'sc, Expression<'sc>> {
+pub fn desugar_match_expression(
+    primary_expression: Expression,
+    branches: Vec<MatchBranch>,
+    _span: Span,
+) -> CompileResult<Expression> {
     let mut errors = vec![];
     let mut warnings = vec![];
 
@@ -1506,7 +1539,8 @@ pub fn desugar_match_expression<'sc>(
                     "found None",
                     branch_span.clone(),
                 )];
-                let exp = Expression::Unit {
+                let exp = Expression::Tuple {
+                    fields: vec![],
                     span: branch_span.clone(),
                 };
                 return ok(exp, vec![], errors);
@@ -1557,7 +1591,7 @@ pub fn desugar_match_expression<'sc>(
                 type_ascription: TypeInfo::Unknown,
                 type_ascription_span: None,
             });
-            let new_span = join_spans(left_impl.span.clone(), right_impl.span());
+            let new_span = join_spans(left_impl.span().clone(), right_impl.span());
             code_block_stmts.push(AstNode {
                 content: AstNodeContent::Declaration(decl),
                 span: new_span.clone(),
@@ -1670,7 +1704,8 @@ pub fn desugar_match_expression<'sc>(
                     if_statement.span(),
                 ));
                 // construct unit expression for error recovery
-                let exp = Expression::Unit {
+                let exp = Expression::Tuple {
+                    fields: vec![],
                     span: if_statement.span(),
                 };
                 return ok(exp, warnings, errors);
