@@ -8,7 +8,7 @@ pub enum Visibility {
 }
 
 impl Visibility {
-    pub(crate) fn parse_from_pair(input: Pair<'_, Rule>) -> Self {
+    pub(crate) fn parse_from_pair(input: Pair<Rule>) -> Self {
         match input.as_str().trim() {
             "pub" => Visibility::Public,
             _ => Visibility::Private,
