@@ -31,7 +31,7 @@ pub enum TypeInfo {
     /// For the type inference engine to use when a type references another type
     Ref(TypeId),
 
-    Unit,
+    Tuple(Vec<TypeId>),
     /// Represents a type which contains methods to issue a contract call.
     /// The specific contract is identified via the `Ident` within.
     ContractCaller {
@@ -40,7 +40,7 @@ pub enum TypeInfo {
         address: String,
         // TODO(static span): the above String should be a TypedExpression
         //        #[derivative(PartialEq = "ignore", Hash = "ignore")]
-        //        address: Box<TypedExpression<'sc>>,
+        //        address: Box<TypedExpression>,
     },
     /// A custom type could be a struct or similar if the name is in scope,
     /// or just a generic parameter if it is not.
@@ -69,10 +69,10 @@ impl Default for TypeInfo {
 }
 
 impl TypeInfo {
-    pub(crate) fn parse_from_pair<'sc>(
-        input: Pair<'sc, Rule>,
+    pub(crate) fn parse_from_pair(
+        input: Pair<Rule>,
         config: Option<&BuildConfig>,
-    ) -> CompileResult<'sc, Self> {
+    ) -> CompileResult<Self> {
         match input.as_rule() {
             Rule::type_name | Rule::generic_type_param => (),
             _ => {
@@ -90,10 +90,10 @@ impl TypeInfo {
         Self::parse_from_pair_inner(input.into_inner().next().unwrap(), config)
     }
 
-    fn parse_from_pair_inner<'sc>(
-        input: Pair<'sc, Rule>,
+    fn parse_from_pair_inner(
+        input: Pair<Rule>,
         config: Option<&BuildConfig>,
-    ) -> CompileResult<'sc, Self> {
+    ) -> CompileResult<Self> {
         let mut warnings = vec![];
         let mut errors = vec![];
         let span = Span {
@@ -115,7 +115,7 @@ impl TypeInfo {
                 "u32" => TypeInfo::UnsignedInteger(IntegerBits::ThirtyTwo),
                 "u64" => TypeInfo::UnsignedInteger(IntegerBits::SixtyFour),
                 "bool" => TypeInfo::Boolean,
-                "unit" => TypeInfo::Unit,
+                "unit" => TypeInfo::Tuple(Vec::new()),
                 "byte" => TypeInfo::Byte,
                 "b256" => TypeInfo::B256,
                 "Self" | "self" => TypeInfo::SelfType,
@@ -193,7 +193,20 @@ impl TypeInfo {
                 };
                 TypeInfo::Array(insert_type(elem_type_info), elem_count)
             }
-            Rule::unit => TypeInfo::Unit,
+            Rule::tuple_type => {
+                let mut field_type_ids = vec![];
+                for field in input.into_inner() {
+                    let field_type = check!(
+                        TypeInfo::parse_from_pair(field, config),
+                        TypeInfo::Tuple(Vec::new()),
+                        warnings,
+                        errors
+                    );
+                    let field_type_id = crate::type_engine::insert_type(field_type);
+                    field_type_ids.push(field_type_id);
+                }
+                TypeInfo::Tuple(field_type_ids)
+            }
             _ => {
                 errors.push(CompileError::Internal(
                     "Unexpected token while parsing inner type.",
@@ -221,7 +234,13 @@ impl TypeInfo {
             Boolean => "bool".into(),
             Custom { name } => format!("unresolved {}", name),
             Ref(id) => format!("T{} ({})", id, (*id).friendly_type_str()),
-            Unit => "()".into(),
+            Tuple(fields) => {
+                let field_strs = fields
+                    .iter()
+                    .map(|field| field.friendly_type_str())
+                    .collect::<Vec<String>>();
+                format!("({})", field_strs.join(", "))
+            }
             SelfType => "Self".into(),
             Byte => "byte".into(),
             B256 => "b256".into(),
@@ -261,7 +280,13 @@ impl TypeInfo {
             Boolean => "bool".into(),
             Custom { name } => format!("unresolved {}", name),
             Ref(id) => format!("T{} ({})", id, (*id).json_abi_str()),
-            Unit => "()".into(),
+            Tuple(fields) => {
+                let field_strs = fields
+                    .iter()
+                    .map(|field| field.json_abi_str())
+                    .collect::<Vec<String>>();
+                format!("({})", field_strs.join(", "))
+            }
             SelfType => "Self".into(),
             Byte => "byte".into(),
             B256 => "b256".into(),
@@ -282,10 +307,7 @@ impl TypeInfo {
     }
 
     /// maps a type to a name that is used when constructing function selectors
-    pub(crate) fn to_selector_name<'sc>(
-        &self,
-        error_msg_span: &Span<'sc>,
-    ) -> CompileResult<'sc, String> {
+    pub(crate) fn to_selector_name(&self, error_msg_span: &Span) -> CompileResult<String> {
         use TypeInfo::*;
         let name = match self {
             Str(len) => format!("str[{}]", len),
@@ -301,7 +323,28 @@ impl TypeInfo {
             }
             Boolean => "bool".into(),
 
-            Unit => "unit".into(),
+            Tuple(fields) => {
+                let field_names = {
+                    let names = fields
+                        .iter()
+                        .map(|field_type| {
+                            resolve_type(*field_type, error_msg_span)
+                                .expect("unreachable?")
+                                .to_selector_name(error_msg_span)
+                        })
+                        .collect::<Vec<CompileResult<String>>>();
+                    let mut buf = vec![];
+                    for name in names {
+                        match name.value {
+                            Some(value) => buf.push(value),
+                            None => return name,
+                        }
+                    }
+                    buf
+                };
+
+                format!("({})", field_names.join(","))
+            }
             Byte => "byte".into(),
             B256 => "b256".into(),
             Struct { fields, .. } => {
@@ -362,10 +405,7 @@ impl TypeInfo {
         ok(name, vec![], vec![])
     }
     /// Calculates the stack size of this type, to be used when allocating stack memory for it.
-    pub(crate) fn size_in_words<'sc>(
-        &self,
-        err_span: &Span<'sc>,
-    ) -> Result<u64, CompileError<'sc>> {
+    pub(crate) fn size_in_words(&self, err_span: &Span) -> Result<u64, CompileError> {
         match self {
             // Each char is a byte, so the size is the num of characters / 8
             // rounded up to the nearest word
@@ -373,7 +413,16 @@ impl TypeInfo {
             // Since things are unpacked, all unsigned integers are 64 bits.....for now
             TypeInfo::UnsignedInteger(_) | TypeInfo::Numeric => Ok(1),
             TypeInfo::Boolean => Ok(1),
-            TypeInfo::Unit => Ok(0),
+            TypeInfo::Tuple(fields) => Ok(fields
+                .iter()
+                .map(|field_type| {
+                    resolve_type(*field_type, err_span)
+                        .expect("should be unreachable?")
+                        .size_in_words(err_span)
+                })
+                .collect::<Result<Vec<u64>, _>>()?
+                .iter()
+                .sum()),
             TypeInfo::Byte => Ok(1),
             TypeInfo::B256 => Ok(4),
             TypeInfo::Enum { variant_types, .. } => {
@@ -417,16 +466,86 @@ impl TypeInfo {
     }
     pub(crate) fn is_copy_type(&self) -> bool {
         match self {
-            TypeInfo::UnsignedInteger(_) | TypeInfo::Boolean | TypeInfo::Unit | TypeInfo::Byte => {
+            TypeInfo::UnsignedInteger(_) | TypeInfo::Boolean | TypeInfo::Byte => true,
+            TypeInfo::Tuple(fields) => fields
+                .iter()
+                .all(|field_type| look_up_type_id(*field_type).is_copy_type()),
+            _ => false,
+        }
+    }
+
+    pub fn is_uninhabited(&self) -> bool {
+        match self {
+            TypeInfo::Enum { variant_types, .. } => variant_types
+                .iter()
+                .all(|variant_type| look_up_type_id(variant_type.r#type).is_uninhabited()),
+            TypeInfo::Struct { fields, .. } => fields
+                .iter()
+                .any(|field| look_up_type_id(field.r#type).is_uninhabited()),
+            TypeInfo::Tuple(fields) => fields
+                .iter()
+                .any(|field_type| look_up_type_id(*field_type).is_uninhabited()),
+            _ => false,
+        }
+    }
+
+    pub fn is_zero_sized(&self) -> bool {
+        match self {
+            TypeInfo::Enum { variant_types, .. } => {
+                let mut found_unit_variant = false;
+                for variant_type in variant_types {
+                    let type_info = look_up_type_id(variant_type.r#type);
+                    if type_info.is_uninhabited() {
+                        continue;
+                    }
+                    if type_info.is_zero_sized() && !found_unit_variant {
+                        found_unit_variant = true;
+                        continue;
+                    }
+                    return false;
+                }
                 true
             }
+            TypeInfo::Struct { fields, .. } => {
+                let mut all_zero_sized = true;
+                for field in fields {
+                    let type_info = look_up_type_id(field.r#type);
+                    if type_info.is_uninhabited() {
+                        return true;
+                    }
+                    if !type_info.is_zero_sized() {
+                        all_zero_sized = false;
+                    }
+                }
+                all_zero_sized
+            }
+            TypeInfo::Tuple(fields) => {
+                let mut all_zero_sized = true;
+                for field in fields {
+                    let field_type = look_up_type_id(*field);
+                    if field_type.is_uninhabited() {
+                        return true;
+                    }
+                    if !field_type.is_zero_sized() {
+                        all_zero_sized = false;
+                    }
+                }
+                all_zero_sized
+            }
             _ => false,
         }
     }
 
-    pub(crate) fn matches_type_parameter<'sc>(
+    pub fn is_unit(&self) -> bool {
+        match self {
+            TypeInfo::Tuple(fields) => fields.is_empty(),
+            _ => false,
+        }
+    }
+
+    pub(crate) fn matches_type_parameter(
         &self,
-        mapping: &[(TypeParameter<'sc>, TypeId)],
+        mapping: &[(TypeParameter, TypeId)],
     ) -> Option<TypeId> {
         use TypeInfo::*;
         match self {
@@ -485,12 +604,40 @@ impl TypeInfo {
             TypeInfo::Array(ary_ty_id, count) => look_up_type_id(*ary_ty_id)
                 .matches_type_parameter(mapping)
                 .map(|matching_id| insert_type(TypeInfo::Array(matching_id, *count))),
+            TypeInfo::Tuple(fields) => {
+                let mut new_fields = Vec::new();
+                let mut index = 0;
+                while index < fields.len() {
+                    let new_field_id_opt =
+                        look_up_type_id(fields[index]).matches_type_parameter(mapping);
+                    if let Some(new_field_id) = new_field_id_opt {
+                        new_fields.extend(fields[..index].iter().cloned());
+                        new_fields.push(insert_type(TypeInfo::Ref(new_field_id)));
+                        index += 1;
+                        break;
+                    }
+                    index += 1;
+                }
+                while index < fields.len() {
+                    let new_field =
+                        match look_up_type_id(fields[index]).matches_type_parameter(mapping) {
+                            Some(new_field_id) => insert_type(TypeInfo::Ref(new_field_id)),
+                            None => fields[index],
+                        };
+                    new_fields.push(new_field);
+                    index += 1;
+                }
+                if new_fields.is_empty() {
+                    None
+                } else {
+                    Some(insert_type(TypeInfo::Tuple(new_fields)))
+                }
+            }
             Unknown
             | Str(..)
             | UnsignedInteger(..)
             | Boolean
             | Ref(..)
-            | Unit
             | ContractCaller { .. }
             | SelfType
             | Byte
