@@ -2,7 +2,7 @@ use crate::span::Span;
 
 /// This panics if the spans are not from the same file. This should
 /// only be used on spans that are actually next to each other.
-pub fn join_spans<'sc>(s1: Span<'sc>, s2: Span<'sc>) -> Span<'sc> {
+pub fn join_spans(s1: Span, s2: Span) -> Span {
     assert!(
         s1.input() == s2.input() && s1.path == s2.path,
         "Spans from different files cannot be joined.",
